package test.hds.index;

import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IHDSIndex;
import org.netkernel.layer0.representation.IHDSIndexSpecification;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.IHDSPredicate;
import org.netkernel.layer0.representation.impl.HDSIndex;
import org.netkernel.layer0.representation.impl.HDSIndexSpecificationFactory;
import org.netkernel.layer0.representation.impl.HDSPredicateFactory;
import org.netkernel.layer0.util.HDSUtils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class HDSIndexTestAccessor extends StandardAccessorImpl
{
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		int testId=Integer.parseInt(aContext.getThisRequest().getArgumentValue("test-id"));
		aContext.setCWU("res:/test/hds/index/");
		switch(testId)
		{
		case 1:
			onTest1(aContext);
			break;
		case 2:
			onTest2(aContext);
			break;
		case 3:
			onTest3(aContext);
			break;
		case 4:
			onTest4(aContext);
			break;
		case 5:
			onTest5(aContext);
			break;
		case 6:
			onTest6(aContext);
			break;
		case 7:
			onTest7(aContext);
			break;
		}
	}
	
	private void onTest1(INKFRequestContext aContext) throws Exception
	{
		IHDSNode tree=aContext.source("testTree.xml",IHDSNode.class);
		IHDSIndex index=new HDSIndex(tree,HDSIndexSpecificationFactory.INDEX_ALL_BY_VALUE);
		
		IHDSNodeList nodes=index.get("1");		
		aContext.createResponseFrom(nodes.size());
	}

	private void onTest2(INKFRequestContext aContext) throws Exception
	{
		INKFRequest req=aContext.createRequest("active:indexHDS");
		req.addArgument("operand", "testTree.xml");
		IHDSIndexSpecification indexSpec=HDSIndexSpecificationFactory.INDEX_ALL_BY_VALUE;
		req.addArgumentByValue("operator", indexSpec);
		
		IHDSIndex index=(IHDSIndex)aContext.issueRequest(req);
		IHDSNodeList nodes=index.get("1");		
		aContext.createResponseFrom(nodes.size());
	}

	private void onTest3(INKFRequestContext aContext) throws Exception
	{
		IHDSNode tree=aContext.source("testTree.xml",IHDSNode.class);
		IHDSNode indexRoot=tree.getFirstNode("/a/d/d[2]");
		
		INKFRequest req=aContext.createRequest("active:indexHDS");
		req.addArgumentByValue("operand", indexRoot);
		IHDSIndexSpecification indexSpec=HDSIndexSpecificationFactory.INDEX_ALL_BY_VALUE;
		req.addArgumentByValue("operator", indexSpec);

		IHDSIndex index=(IHDSIndex)aContext.issueRequest(req);
		IHDSNodeList nodes=index.get("1");		
		aContext.createResponseFrom(nodes.size());
	
	}

	private void onTest4(INKFRequestContext aContext) throws Exception
	{
		INKFRequest req=aContext.createRequest("active:indexHDS");
		req.addArgument("operand", "testTree.xml");
		
		IHDSIndexSpecification indexSpec=new IHDSIndexSpecification()
		{	public Object getKeyValue(IHDSNode nodeToIndex)
			{	return nodeToIndex.getValue();
			}

			public IHDSNodeList getNodesToIndex(IHDSNode contextNode)
			{	IHDSPredicate filter = new IHDSPredicate()
				{	public boolean matches(IHDSNode node)
					{	try
						{	return Integer.parseInt((String)node.getValue())>1;
						}
						catch (NumberFormatException e)
						{	return false;
						}
					}
				};
				return HDSUtils.filteredInOrderTraversal(contextNode,filter);
			}
		};
		
		req.addArgumentByValue("operator", indexSpec);
		
		IHDSIndex index=(IHDSIndex)aContext.issueRequest(req);
		IHDSNodeList nodes=index.get("2");		
		aContext.createResponseFrom(nodes.size());
	}

	private void onTest5(INKFRequestContext aContext) throws Exception
	{
		INKFRequest req=aContext.createRequest("active:indexHDS");
		req.addArgument("operand", "testTree.xml");
		IHDSIndexSpecification indexSpec=HDSIndexSpecificationFactory.INDEX_ALL_BY_VALUE;
		req.addArgumentByValue("operator", indexSpec);
		
		IHDSIndex index=(IHDSIndex)aContext.issueRequest(req);
		IHDSNodeList nodes=index.get(null);
		aContext.createResponseFrom(nodes.size());
	}

	private void onTest6(INKFRequestContext aContext) throws Exception
	{
		INKFRequest req=aContext.createRequest("active:indexHDS");
		req.addArgument("operand", "testTree2.xml");
		
		IHDSIndexSpecification indexSpec=new IHDSIndexSpecification()
		{	public Object getKeyValue(IHDSNode nodeToIndex)
			{	return nodeToIndex.getFirstValue("type");
			}
			public IHDSNodeList getNodesToIndex(IHDSNode contextNode)
			{	return contextNode.getNodes("/things/thing");
			}
		};
		req.addArgumentByValue("operator", indexSpec);
		
		IHDSIndex index=(IHDSIndex)aContext.issueRequest(req);
		IHDSNodeList nodes=index.get("ok");
		StringBuilder sb=new StringBuilder();
		for (IHDSNode n: nodes)
		{	String id=n.getFirstValue("id").toString();
			sb.append(id);
			sb.append("_");
		}
		
		aContext.createResponseFrom(sb.toString());
	}

	private void onTest7(INKFRequestContext aContext) throws Exception
	{
		INKFRequest req=aContext.createRequest("active:indexHDS");
		req.addArgument("operand", "testTree.xml");
		IHDSIndex index=(IHDSIndex)aContext.issueRequest(req);
		
		IHDSNodeList nodes=index.get("1");		
		aContext.createResponseFrom(nodes.size());
	}

	
}
